package com.example.backend.books.service;

import java.util.List;
import java.util.Objects;
import java.util.Optional;

import org.springframework.stereotype.Service;

import com.example.backend.books.model.BookEntity;
import com.example.backend.books.repository.BookRepository;
import com.example.backend.core.error.NotFoundException;

@Service
public class BookService {
    private final BookRepository repository;

    public BookService(BookRepository repository) {
        this.repository = repository;
    }

    public List<BookEntity> getAll(Long categoryId) {
        if (Objects.equals(categoryId, 0L)) {
            return repository.getAll();
        }
        return repository.getAll().stream()
                .filter(book -> book.getCategory().getId().equals(categoryId))
                .toList();
    }

    public BookEntity get(Long id) {
        return Optional.ofNullable(repository.get(id))
                .orElseThrow(() -> new NotFoundException(id));
    }

    public BookEntity create(BookEntity entity) {
        return repository.create(entity);
    }

    public BookEntity update(Long id, BookEntity entity) {
        final BookEntity existsEntity = get(id);
        existsEntity.setCategory(entity.getCategory());
        existsEntity.setTitle(entity.getTitle());
        existsEntity.setAuthor(entity.getAuthor());
        existsEntity.setPrice(entity.getPrice());
        existsEntity.setCount(entity.getCount());
        existsEntity.setDescription(entity.getDescription());
        existsEntity.setAnnotation(entity.getAnnotation());
        existsEntity.setBookCode(entity.getBookCode());
        existsEntity.setPublisher(entity.getPublisher());
        existsEntity.setSeries(entity.getSeries());
        existsEntity.setPublicationYear(entity.getPublicationYear());
        existsEntity.setPagesNum(entity.getPagesNum());
        existsEntity.setSize(entity.getSize());
        existsEntity.setCoverType(entity.getCoverType());
        existsEntity.setCirculation(entity.getCirculation());
        existsEntity.setWeight(entity.getWeight());
        existsEntity.setImage(entity.getImage());
        return repository.update(existsEntity);
    }

    public BookEntity delete(Long id) {
        final BookEntity existsEntity = get(id);
        return repository.delete(existsEntity);
    }
}
