package com.example.backend.categories.service;

import java.util.List;
import java.util.Optional;

import org.springframework.stereotype.Service;

import com.example.backend.categories.model.CategoryEntity;
import com.example.backend.categories.repository.CategoryRepository;
import com.example.backend.core.error.NotFoundException;

@Service
public class CategoryService {
    private final CategoryRepository repository;

    public CategoryService(CategoryRepository repository) {
        this.repository = repository;
    }

    public List<CategoryEntity> getAll() {
        return repository.getAll();
    }

    public CategoryEntity get(Long id) {
        return Optional.ofNullable(repository.get(id))
                .orElseThrow(() -> new NotFoundException(id));
    }

    public CategoryEntity create(CategoryEntity entity) {
        return repository.create(entity);
    }

    public CategoryEntity update(Long id, CategoryEntity entity) {
        final CategoryEntity existsEntity = get(id);
        existsEntity.setName(entity.getName());
        return repository.update(existsEntity);
    }

    public CategoryEntity delete(Long id) {
        final CategoryEntity existsEntity = get(id);
        return repository.delete(existsEntity);
    }
}
