package com.example.backend.orders.api;

import java.text.ParseException;
import java.util.List;

import org.modelmapper.ModelMapper;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.example.backend.orders.model.OrderEntity;
import com.example.backend.orders.service.OrderService;
import com.example.backend.users.service.UserService;
import com.example.backend.core.configuration.Constants;
import com.example.backend.core.utils.Formatter;

import jakarta.validation.Valid;

@RestController
@RequestMapping(Constants.API_URL + "/order")
public class OrderController {
    private final OrderService orderService;
    private final UserService userService;
    private final ModelMapper modelMapper;

    public OrderController(OrderService orderService, UserService userService, ModelMapper modelMapper) {
        this.orderService = orderService;
        this.userService = userService;
        this.modelMapper = modelMapper;
    }

    private OrderDto toDto(OrderEntity entity) {
        final OrderDto dto = modelMapper.map(entity, OrderDto.class);
        dto.setDate(Formatter.format(entity.getDate()));
        dto.setSum(orderService.getFullSum(entity.getId()));
        return dto;
    }

    private OrderEntity toEntity(OrderDto dto) throws ParseException {
        final OrderEntity entity = modelMapper.map(dto, OrderEntity.class);
        entity.setUser(userService.get(dto.getUserId()));
        entity.setDate(Formatter.parse(dto.getDate()));
        return entity;
    }

    @GetMapping
    public List<OrderDto> getAll(@RequestParam(name = "userId", defaultValue = "0") Long userId) {
        return orderService.getAll(userId).stream().map(this::toDto).toList();
    }

    @GetMapping("/{id}")
    public OrderDto get(@PathVariable(name = "id") Long id) {
        return toDto(orderService.get(id));
    }

    @PostMapping
    public OrderDto create(@RequestBody @Valid OrderDto dto) throws ParseException {
        return toDto(orderService.create(toEntity(dto)));
    }

    @PutMapping("/{id}")
    public OrderDto update(@PathVariable(name = "id") Long id, @RequestBody OrderDto dto) throws ParseException {
        return toDto(orderService.update(id, toEntity(dto)));
    }

    @DeleteMapping("/{id}")
    public OrderDto delete(@PathVariable(name = "id") Long id) {
        return toDto(orderService.delete(id));
    }
}
