package com.example.backend.orders.service;

import java.util.List;
import java.util.Objects;
import java.util.Optional;

import org.springframework.stereotype.Service;

import com.example.backend.orders.model.OrderEntity;
import com.example.backend.orders.repository.OrderRepository;
import com.example.backend.ordersbooks.service.OrderBookService;
import com.example.backend.core.error.NotFoundException;

@Service
public class OrderService {
    private final OrderRepository repository;
    private final OrderBookService orderBookService;

    public OrderService(OrderRepository repository, OrderBookService orderBookService) {
        this.repository = repository;
        this.orderBookService = orderBookService;
    }

    public List<OrderEntity> getAll(Long userId) {
        if (Objects.equals(userId, 0L)) {
            return repository.getAll();
        }
        return repository.getAll().stream()
                .filter(order -> order.getUser().getId().equals(userId))
                .toList();
    }

    public OrderEntity get(Long id) {
        return Optional.ofNullable(repository.get(id))
                .orElseThrow(() -> new NotFoundException(id));
    }

    public OrderEntity create(OrderEntity entity) {
        return repository.create(entity);
    }

    public OrderEntity update(Long id, OrderEntity entity) {
        final OrderEntity existsEntity = get(id);
        existsEntity.setStatus(entity.getStatus());
        return repository.update(existsEntity);
    }

    public OrderEntity delete(Long id) {
        final OrderEntity existsEntity = get(id);
        return repository.delete(existsEntity);
    }

    public Double getFullSum(Long id) {
        return orderBookService.getAll(id, 0L).stream()
                .mapToDouble(orderBook -> orderBookService.getSum(orderBook.getId()))
                .sum();
    }
}
