package com.example.backend.ordersbooks.api;

import java.util.List;

import org.modelmapper.ModelMapper;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.example.backend.ordersbooks.model.OrderBookEntity;
import com.example.backend.ordersbooks.service.OrderBookService;
import com.example.backend.orders.service.OrderService;
import com.example.backend.books.service.BookService;
import com.example.backend.core.configuration.Constants;

import jakarta.validation.Valid;

@RestController
@RequestMapping(Constants.API_URL + "/orderbook")
public class OrderBookController {
    private final OrderBookService orderBookService;
    private final OrderService orderService;
    private final BookService bookService;
    private final ModelMapper modelMapper;

    public OrderBookController(OrderBookService orderBookService, OrderService orderService, BookService bookService,
            ModelMapper modelMapper) {
        this.orderBookService = orderBookService;
        this.orderService = orderService;
        this.bookService = bookService;
        this.modelMapper = modelMapper;
    }

    private OrderBookDto toDto(OrderBookEntity entity) {
        return modelMapper.map(entity, OrderBookDto.class);
    }

    private OrderBookEntity toEntity(OrderBookDto dto) {
        final OrderBookEntity entity = modelMapper.map(dto, OrderBookEntity.class);
        entity.setOrder(orderService.get(dto.getOrderId()));
        entity.setBook(bookService.get(dto.getBookId()));
        return entity;
    }

    @GetMapping
    public List<OrderBookDto> getAll(@RequestParam(name = "orderId", defaultValue = "0") Long orderId,
            @RequestParam(name = "bookId", defaultValue = "0") Long bookId) {
        return orderBookService.getAll(orderId, bookId).stream().map(this::toDto).toList();
    }

    @GetMapping("/{id}")
    public OrderBookDto get(@PathVariable(name = "id") Long id) {
        return toDto(orderBookService.get(id));
    }

    @PostMapping
    public OrderBookDto create(@RequestBody @Valid OrderBookDto dto) {
        return toDto(orderBookService.create(toEntity(dto)));
    }

    @PutMapping("/{id}")
    public OrderBookDto update(@PathVariable(name = "id") Long id, @RequestBody OrderBookDto dto) {
        return toDto(orderBookService.update(id, toEntity(dto)));
    }

    @DeleteMapping("/{id}")
    public OrderBookDto delete(@PathVariable(name = "id") Long id) {
        return toDto(orderBookService.delete(id));
    }
}
