package com.example.backend.ordersbooks.service;

import java.util.List;
import java.util.Objects;
import java.util.Optional;

import org.springframework.stereotype.Service;

import com.example.backend.ordersbooks.model.OrderBookEntity;
import com.example.backend.ordersbooks.repository.OrderBookRepository;
import com.example.backend.core.error.NotFoundException;

@Service
public class OrderBookService {
    private final OrderBookRepository repository;

    public OrderBookService(OrderBookRepository repository) {
        this.repository = repository;
    }

    public List<OrderBookEntity> getAll(Long orderId, Long bookId) {
        if (Objects.equals(orderId, 0L) && Objects.equals(bookId, 0L)) {
            return repository.getAll();
        }
        return repository.getAll().stream()
                .filter(order -> order.getOrder().getId().equals(orderId) || order.getBook().getId().equals(bookId))
                .toList();
    }

    public OrderBookEntity get(Long id) {
        return Optional.ofNullable(repository.get(id))
                .orElseThrow(() -> new NotFoundException(id));
    }

    public OrderBookEntity create(OrderBookEntity entity) {
        return repository.create(entity);
    }

    public OrderBookEntity update(Long id, OrderBookEntity entity) {
        final OrderBookEntity existsEntity = get(id);
        existsEntity.setOrder(entity.getOrder());
        existsEntity.setBook(entity.getBook());
        existsEntity.setCount(entity.getCount());
        return repository.update(existsEntity);
    }

    public OrderBookEntity delete(Long id) {
        final OrderBookEntity existsEntity = get(id);
        return repository.delete(existsEntity);
    }

    public Double getSum(Long id) {
        final OrderBookEntity entity = get(id);
        return entity.getCount() * entity.getBook().getPrice();
    }
}
