package com.example.backend;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import com.example.backend.categories.model.CategoryEntity;
import com.example.backend.categories.service.CategoryService;
import com.example.backend.core.error.NotFoundException;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
class CategoryServiceTests {
	@Autowired
	private CategoryService categoryService;

	@Test
	void getTest() {
		Assertions.assertThrows(NotFoundException.class, () -> categoryService.get(0L));
	}

	@Test
	@Order(1)
	void createTest() {
		categoryService.create(new CategoryEntity(null, "Фэнтези"));
		categoryService.create(new CategoryEntity(null, "Художественная литература"));
		final CategoryEntity last = categoryService.create(new CategoryEntity(null, "Книги для подростков"));
		Assertions.assertEquals(3, categoryService.getAll().size());
		Assertions.assertEquals(last, categoryService.get(3L));
	}

	@Test
	@Order(2)
	void updateTest() {
		final String test = "TEST";
		final CategoryEntity entity = categoryService.get(3L);
		final String oldName = entity.getName();
		final CategoryEntity newEntity = categoryService.update(3L, new CategoryEntity(1L, test));
		Assertions.assertEquals(3, categoryService.getAll().size());
		Assertions.assertEquals(newEntity, categoryService.get(3L));
		Assertions.assertEquals(test, newEntity.getName());
		Assertions.assertNotEquals(oldName, newEntity.getName());
	}

	@Test
	@Order(3)
	void deleteTest() {
		categoryService.delete(3L);
		Assertions.assertEquals(2, categoryService.getAll().size());
		final CategoryEntity last = categoryService.get(2L);
		Assertions.assertEquals(2L, last.getId());

		final CategoryEntity newEntity = categoryService.create(new CategoryEntity(null, "Книги для подростков"));
		Assertions.assertEquals(3, categoryService.getAll().size());
		Assertions.assertEquals(4L, newEntity.getId());
	}
}
