package com.example.backend;

import java.text.ParseException;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import com.example.backend.users.model.UserEntity;
import com.example.backend.users.service.UserService;
import com.example.backend.categories.model.CategoryEntity;
import com.example.backend.categories.service.CategoryService;
import com.example.backend.orders.model.OrderEntity;
import com.example.backend.orders.service.OrderService;
import com.example.backend.books.model.BookEntity;
import com.example.backend.books.service.BookService;
import com.example.backend.ordersbooks.model.OrderBookEntity;
import com.example.backend.ordersbooks.service.OrderBookService;
import com.example.backend.core.error.NotFoundException;
import com.example.backend.core.utils.Formatter;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
class OrderBookServiceTests {
    @Autowired
    private UserService userService;

    @Autowired
    private CategoryService categoryService;

    @Autowired
    private OrderService orderService;

    @Autowired
    private BookService bookService;

    @Autowired
    private OrderBookService orderBookService;

    @Test
    void getTest() {
        Assertions.assertThrows(NotFoundException.class, () -> orderBookService.get(0L));
    }

    @Test
    @Order(1)
    void createTest() throws ParseException {
        final var category = categoryService.create(new CategoryEntity(null, "Художественная литература"));
        final var user = userService
                .create(new UserEntity(null, "TestUser", "user53262@gmail.com", "lawy7728", false));
        final var order1 = orderService
                .create(new OrderEntity(null, user, Formatter.parse("2024-03-28"), "Выполняется"));
        final var order2 = orderService
                .create(new OrderEntity(null, user, Formatter.parse("2022-09-07"), "Выдан"));
        final var book1 = bookService.create(new BookEntity(null, category, "Шестерка воронов", "Ли Бардуго",
                700.00, 5624,
                "Триша Левенселлер — американская писательница, подарившая вам роман «Тени между нами», который стал бестселлером Amazon в разделе «Фэнтези YA», вновь радует своих фанатов новой магической историей «Клинок тайн».\n\nЕще до выхода в свет книгу добавили в раздел «Хочу прочитать» более 10 тысяч человек на портале Goodreads.com.\n\n«Клинок тайн» — первая часть захватывающей дилогии про девушку, которая благодаря магии крови создает самое опасное в мире оружие, и ослепительного юношу, что составляет ей компанию в этом непростом приключении.",
                "Юная Зива предпочитает проводить время с оружием, а не с людьми. Она кузнец и ей предстоит создать самое могущественное оружие на земле.\n\nДевушка демонстрирует меч, способный от одной капли крови врага раскрыть все тайные замыслы его обладателю.\n\nОднако у заказчика на оружие другие планы: заставить Зиву вооружить мечами свое войско и поработить весь мир. Когда девушка понимает, что меч, созданный благодаря магии крови, невозможно уничтожить и рано или поздно на нее объявят охоту, ей остается только одно – бежать, не оглядываясь. Зиве теперь суждено спасти мир от собственного творения. Но как бы далеко она ни старалась убежать, все ловушки уже расставлены.",
                "2946447", "Эксмо", "Young Adult. Бестселлеры Триши Левенселлер", 2022, 480,
                "20.7x13x2.5",
                "Мягкий переплёт", 1500, 440, ""));
        final var book2 = bookService.create(new BookEntity(null, category, "Форсайт", "Сергей Лукъяненко",
                775.00,
                1240,
                "Новый долгожданный роман от автора бестселлеров «Ночной дозор» и «Черновик». Увлекательная история о Мире После и мире настоящего, где 5 процентов людей знают о надвигающемся апокалипсисе больше, чем кто-либо может представить.",
                "Людям порой снится прошлое. Иногда хорошее, иногда не очень. Но что делать, если тебе начинает сниться будущее? И в нём ничего хорошего нет совсем.",
                "3001249", "АСТ", "Книги Сергея Лукьяненко", 2023, 352, "20.5x13x2", "Твердый переплёт",
                20000,
                350, ""));

        orderBookService.create(new OrderBookEntity(null, order1, book1, 4));
        orderBookService.create(new OrderBookEntity(null, order1, book2, 1));
        final OrderBookEntity last = orderBookService.create(new OrderBookEntity(null, order2, book1, 2));
        Assertions.assertEquals(3, orderBookService.getAll(0L, 0L).size());
        Assertions.assertEquals(last, orderBookService.get(3L));
    }

    @Test
    @Order(2)
    void updateTest() throws ParseException {
        final var category = categoryService.create(new CategoryEntity(null, "Художественная литература"));
        final var user = userService
                .create(new UserEntity(null, "TestUser", "user53262@gmail.com", "lawy7728", false));
        final var order1 = orderService
                .create(new OrderEntity(null, user, Formatter.parse("2024-03-28"), "Выполняется"));
        final var book2 = bookService.create(new BookEntity(null, category, "Форсайт", "Сергей Лукъяненко",
                775.00,
                1240,
                "Новый долгожданный роман от автора бестселлеров «Ночной дозор» и «Черновик». Увлекательная история о Мире После и мире настоящего, где 5 процентов людей знают о надвигающемся апокалипсисе больше, чем кто-либо может представить.",
                "Людям порой снится прошлое. Иногда хорошее, иногда не очень. Но что делать, если тебе начинает сниться будущее? И в нём ничего хорошего нет совсем.",
                "3001249", "АСТ", "Книги Сергея Лукьяненко", 2023, 352, "20.5x13x2", "Твердый переплёт",
                20000,
                350, ""));
        final Integer newCount = 6;
        final OrderBookEntity entity = orderBookService.get(3L);
        final Integer oldCount = entity.getCount();
        final OrderBookEntity newEntity = orderBookService.update(3L,
                new OrderBookEntity(null, order1, book2, newCount));
        Assertions.assertEquals(3, orderBookService.getAll(0L, 0L).size());
        Assertions.assertEquals(newEntity, orderBookService.get(3L));
        Assertions.assertEquals(newCount, newEntity.getCount());
        Assertions.assertNotEquals(oldCount, newEntity.getCount());
    }

    @Test
    @Order(3)
    void deleteTest() throws ParseException {
        final var category = categoryService.create(new CategoryEntity(null, "Художественная литература"));
        final var user = userService
                .create(new UserEntity(null, "TestUser", "user53262@gmail.com", "lawy7728", false));
        final var order2 = orderService
                .create(new OrderEntity(null, user, Formatter.parse("2022-09-07"), "Выдан"));
        final var book1 = bookService.create(new BookEntity(null, category, "Шестерка воронов", "Ли Бардуго",
                700.00, 5624,
                "Триша Левенселлер — американская писательница, подарившая вам роман «Тени между нами», который стал бестселлером Amazon в разделе «Фэнтези YA», вновь радует своих фанатов новой магической историей «Клинок тайн».\n\nЕще до выхода в свет книгу добавили в раздел «Хочу прочитать» более 10 тысяч человек на портале Goodreads.com.\n\n«Клинок тайн» — первая часть захватывающей дилогии про девушку, которая благодаря магии крови создает самое опасное в мире оружие, и ослепительного юношу, что составляет ей компанию в этом непростом приключении.",
                "Юная Зива предпочитает проводить время с оружием, а не с людьми. Она кузнец и ей предстоит создать самое могущественное оружие на земле.\n\nДевушка демонстрирует меч, способный от одной капли крови врага раскрыть все тайные замыслы его обладателю.\n\nОднако у заказчика на оружие другие планы: заставить Зиву вооружить мечами свое войско и поработить весь мир. Когда девушка понимает, что меч, созданный благодаря магии крови, невозможно уничтожить и рано или поздно на нее объявят охоту, ей остается только одно – бежать, не оглядываясь. Зиве теперь суждено спасти мир от собственного творения. Но как бы далеко она ни старалась убежать, все ловушки уже расставлены.",
                "2946447", "Эксмо", "Young Adult. Бестселлеры Триши Левенселлер", 2022, 480,
                "20.7x13x2.5",
                "Мягкий переплёт", 1500, 440, ""));

        orderBookService.delete(3L);
        Assertions.assertEquals(2, orderBookService.getAll(0L, 0L).size());
        final OrderBookEntity last = orderBookService.get(2L);
        Assertions.assertEquals(2L, last.getId());

        final OrderBookEntity newEntity = orderBookService.create(new OrderBookEntity(null, order2, book1, 2));
        Assertions.assertEquals(3, orderBookService.getAll(0L, 0L).size());
        Assertions.assertEquals(4L, newEntity.getId());
    }

    @Test
    @Order(4)
    void getSumTest() throws ParseException {
        final var category = categoryService.create(new CategoryEntity(null, "Художественная литература"));
        final var user = userService
                .create(new UserEntity(null, "TestUser", "user53262@gmail.com", "lawy7728", false));
        final var order1 = orderService
                .create(new OrderEntity(null, user, Formatter.parse("2024-03-28"), "Выполняется"));
        final var book2 = bookService.create(new BookEntity(null, category, "Форсайт", "Сергей Лукъяненко",
                775.00,
                1240,
                "Новый долгожданный роман от автора бестселлеров «Ночной дозор» и «Черновик». Увлекательная история о Мире После и мире настоящего, где 5 процентов людей знают о надвигающемся апокалипсисе больше, чем кто-либо может представить.",
                "Людям порой снится прошлое. Иногда хорошее, иногда не очень. Но что делать, если тебе начинает сниться будущее? И в нём ничего хорошего нет совсем.",
                "3001249", "АСТ", "Книги Сергея Лукьяненко", 2023, 352, "20.5x13x2", "Твердый переплёт",
                20000,
                350, ""));

        final var entity = orderBookService.get(1L);
        Double initialSum = orderBookService.getSum(1L);
        Assertions.assertEquals(entity.getCount() * entity.getBook().getPrice(), initialSum);
        final var changedEntity = orderBookService.update(1L, new OrderBookEntity(null, order1, book2, 6));
        Double changedSum = orderBookService.getSum(1L);
        Assertions.assertEquals(changedEntity.getCount() * changedEntity.getBook().getPrice(), changedSum);
        Assertions.assertNotEquals(initialSum, changedSum);
    }
}
