package com.example.backend;

import java.text.ParseException;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import com.example.backend.categories.model.CategoryEntity;
import com.example.backend.categories.service.CategoryService;
import com.example.backend.books.model.BookEntity;
import com.example.backend.books.service.BookService;
import com.example.backend.ordersbooks.model.OrderBookEntity;
import com.example.backend.ordersbooks.service.OrderBookService;
import com.example.backend.orders.model.OrderEntity;
import com.example.backend.orders.service.OrderService;
import com.example.backend.users.model.UserEntity;
import com.example.backend.users.service.UserService;
import com.example.backend.core.error.NotFoundException;
import com.example.backend.core.utils.Formatter;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
class OrderServiceTests {
    @Autowired
    private CategoryService categoryService;

    @Autowired
    private BookService bookService;

    @Autowired
    private OrderBookService orderBookService;

    @Autowired
    private UserService userService;

    @Autowired
    private OrderService orderService;

    @Test
    void getTest() {
        Assertions.assertThrows(NotFoundException.class, () -> orderService.get(0L));
    }

    @Test
    @Order(1)
    void createTest() throws ParseException {
        final UserEntity user = userService.create(
                new UserEntity(null, "TestUser", "user53262@gmail.com", "lawy7728", false));

        orderService.create(new OrderEntity(null, user, Formatter.parse("2024-03-28"), "Выполняется"));
        orderService.create(new OrderEntity(null, user, Formatter.parse("2022-09-07"), "Выдан"));
        final OrderEntity last = orderService
                .create(new OrderEntity(null, user, Formatter.parse("2024-03-15"), "Готов"));
        Assertions.assertEquals(3, orderService.getAll(0L).size());
        Assertions.assertEquals(last, orderService.get(3L));
    }

    @Test
    @Order(2)
    void updateTest() throws ParseException {
        final UserEntity user = userService.create(
                new UserEntity(null, "Chief", "forum98761@gmail.com", "bth4323", true));
        final String test = "TEST";
        final OrderEntity entity = orderService.get(3L);
        final String oldStatus = entity.getStatus();
        final OrderEntity newEntity = orderService.update(3L,
                new OrderEntity(null, user, Formatter.parse("2024-03-15"), test));
        Assertions.assertEquals(3, orderService.getAll(0L).size());
        Assertions.assertEquals(newEntity, orderService.get(3L));
        Assertions.assertEquals(test, newEntity.getStatus());
        Assertions.assertNotEquals(oldStatus, newEntity.getStatus());
    }

    @Test
    @Order(3)
    void deleteTest() throws ParseException {
        orderService.delete(3L);
        Assertions.assertEquals(2, orderService.getAll(0L).size());
        final OrderEntity last = orderService.get(2L);
        Assertions.assertEquals(2L, last.getId());

        final UserEntity user = userService.create(
                new UserEntity(null, "TestUser", "user53262@gmail.com", "lawy7728", false));
        final OrderEntity newEntity = orderService.create(
                new OrderEntity(null, user, Formatter.parse("2024-03-15"), "Готов"));
        Assertions.assertEquals(3, orderService.getAll(0L).size());
        Assertions.assertEquals(4L, newEntity.getId());
    }

    @Test
    @Order(4)
    void getFullSumTest() {
        final var category = categoryService.create(new CategoryEntity(null, "Художественная литература"));
        final var book = bookService.create(new BookEntity(null, category, "Шестерка воронов", "Ли Бардуго",
                700.00, 5624,
                "Триша Левенселлер — американская писательница, подарившая вам роман «Тени между нами», который стал бестселлером Amazon в разделе «Фэнтези YA», вновь радует своих фанатов новой магической историей «Клинок тайн».\n\nЕще до выхода в свет книгу добавили в раздел «Хочу прочитать» более 10 тысяч человек на портале Goodreads.com.\n\n«Клинок тайн» — первая часть захватывающей дилогии про девушку, которая благодаря магии крови создает самое опасное в мире оружие, и ослепительного юношу, что составляет ей компанию в этом непростом приключении.",
                "Юная Зива предпочитает проводить время с оружием, а не с людьми. Она кузнец и ей предстоит создать самое могущественное оружие на земле.\n\nДевушка демонстрирует меч, способный от одной капли крови врага раскрыть все тайные замыслы его обладателю.\n\nОднако у заказчика на оружие другие планы: заставить Зиву вооружить мечами свое войско и поработить весь мир. Когда девушка понимает, что меч, созданный благодаря магии крови, невозможно уничтожить и рано или поздно на нее объявят охоту, ей остается только одно – бежать, не оглядываясь. Зиве теперь суждено спасти мир от собственного творения. Но как бы далеко она ни старалась убежать, все ловушки уже расставлены.",
                "2946447", "Эксмо", "Young Adult. Бестселлеры Триши Левенселлер", 2022, 480, "20.7x13x2.5",
                "Мягкий переплёт", 1500, 440, ""));

        Double initialSum = orderService.getFullSum(1L);
        Assertions.assertEquals(0, initialSum);
        final var orderBook = orderBookService.create(new OrderBookEntity(null, orderService.get(1L), book, 4));
        Double сhangedSum = orderService.getFullSum(1L);
        Assertions.assertEquals(book.getPrice() * orderBook.getCount(), сhangedSum);
        Assertions.assertNotEquals(initialSum, сhangedSum);
    }
}
